use super::tim::types::Encoding;
use serde::{Deserialize, Serialize};
use std::path::PathBuf;

#[derive(Serialize, Deserialize)]
pub struct FileSettings {
    pub compress:     bool,
    pub transparency: Transparency,
    pub encoding:     Encoding,
}

impl FileSettings {
    pub fn compatible(encoding: Encoding) -> FileSettings {
        let mut new_type = FileSettings::default();

        new_type.encoding = encoding;
        new_type
    }
}

impl std::default::Default for FileSettings {
    fn default() -> Self {
        Self{compress: Default::default(), transparency: Transparency::None, encoding: Encoding::FullColor}
    }
}

#[derive(Serialize, Deserialize)]
pub struct Job {
    pub name:         String,
    pub file_path:    PathBuf,
    pub image_pos:    ImagePosition,
    pub palette_rect: Option<PaletteRect>,
    pub settings:     FileSettings,
}

impl Job {
    pub fn new(name: String, file_path: PathBuf, image_pos: (u16, u16), settings: FileSettings) -> Job {
        Job{name, file_path, image_pos: ImagePosition{x: image_pos.0, y: image_pos.1}, palette_rect: None, settings}
    }

    pub fn create_file_path<T:?Sized + std::convert::AsRef<std::ffi::OsStr>>(file_path: PathBuf, location_path: &T) -> PathBuf {
        if file_path.is_file() {
            file_path
        }

        else {
            let mut new_file_path = PathBuf::from(location_path);

            new_file_path.pop();
            new_file_path.push(file_path);
            new_file_path
        }
    }
}

#[derive(Serialize, Deserialize)]
pub struct Project {
    pub jobs: Vec<Job>
}

impl Project {
    pub fn new(jobs: Vec<Job>) -> Project {
        Project{jobs}
    }
}

#[derive(Serialize, Deserialize)]
pub struct ImagePosition {
    pub x: u16,
    pub y: u16,
}

impl ImagePosition {
    pub fn new(x: u16, y: u16) -> ImagePosition {
        ImagePosition{x, y}
    }
}

impl std::default::Default for ImagePosition {
    fn default() -> Self {
        ImagePosition::new(0, 0)
    }
}

#[derive(Clone, Copy, Serialize, Deserialize)]
pub struct ImageSize {
    pub width:  u16,
    pub height: u16,
}

impl ImageSize {
    pub fn new(width: u16, height: u16) -> ImageSize {
        ImageSize{width, height}
    }
}

impl From<ImageSize> for (u16, u16) {
    fn from(value: ImageSize) -> Self {
        (value.width, value.height)
    }
}

impl std::default::Default for ImageSize {
    fn default() -> Self {
        ImageSize::new(0, 0)
    }
}

#[derive(Serialize, Deserialize)]
pub struct PaletteRect {
    pub pos:  ImagePosition,
    pub size: ImageSize,
}

impl PaletteRect {
    pub fn new(x: u16, y: u16, width: u16, height: u16) -> PaletteRect {
        PaletteRect{pos: ImagePosition{x, y}, size: ImageSize{width, height}}
    }
}

impl From<PaletteRect> for (u16, u16, u16, u16) {
    fn from(value: PaletteRect) -> Self {
        (value.pos.x, value.pos.y, value.size.width, value.size.height)
    } 
}

impl std::default::Default for PaletteRect {
    fn default() -> Self {
        PaletteRect{pos: ImagePosition::default(), size: ImageSize::default()}
    }
}

#[derive(Serialize, Deserialize)]
pub enum Transparency {
    None,
    FirstColor,
    PSXSemi,
    Both,
}